//
//   deminor.c
//
//   Copyright 2010 Glibert axel
//
//   This file is part of Deminor.
//
//   Deminor is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Deminor is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Deminor.  If not, see <http://www.gnu.org/licenses/>.
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <SDL.h>
#include <SDL/SDL_ttf.h>
#include <time.h>
#include "easywiz.h"
#include "deminor.h"

/* Initialize game (context) */
void init_game(const int taille, cellule grille[][taille], Uint32 *nbrBomb, GameState *state, Uint32 *offset, Uint32 ellapse, Uint32 score[])
{
    FILE *file = NULL;
    int tmp = 0;
    
    if(*nbrBomb == 0)
    {
        if((file = fopen("config.ini", "r")) == NULL) //sif file doesn't exist
        {
            file = fopen("config.ini", "w");
            fprintf(file, "%ld %ld %ld %ld", 12, 0, 0, 0); //default value
            *nbrBomb = 12;
            score[0] = 0; 
            score[1] = 0; 
            score[2] = 0;
        }
        else
        {
            fscanf(file, "%ld %ld %ld %ld", nbrBomb, &score[0], &score[1], &score[2]);
            if((*nbrBomb) > 99) //security
                *nbrBomb = 99; //I imagine the guy who want try this xD
            if((*nbrBomb) < 1) //min: 1 mine
                *nbrBomb = 1;
        }
        fclose(file);
    }
    genere_grille(taille, grille, *nbrBomb); 
    state->win = false;
    state->end = false;
    state->gamesaved = false;
    state->cellulesDevoilees = 0;
    tmp = *nbrBomb;
    state->reserveFlags = tmp;
    *offset = ellapse;
}

/* Display matrix */
void affich_grille(const int taille, cellule grille[][taille])
{
    int x = 0, y = 0;
    for(x = 0; x < taille; x++)
    {
        for(y = 0; y < taille; y++)
        {
            if(grille[x][y].flag && grille[x][y].hidden)
                wiz_DisplayImagePart(IMG_TILESET, (x*TILE) +TILE, (y*TILE) + TILE, TILE, TILE, TILE*FLAG, 0);
            else if(grille[x][y].flag && !grille[x][y].hidden && grille[x][y].contain == BOMB) //note: j'ai pass une journe entire pour la 3em condition de cette ligne
                wiz_DisplayImagePart(IMG_TILESET, (x*TILE) +TILE, (y*TILE) + TILE, TILE, TILE, TILE*DEMINED, 0);
            else if(grille[x][y].flag && !grille[x][y].hidden && grille[x][y].contain != BOMB) //un drapeau mal plac
                wiz_DisplayImagePart(IMG_TILESET, (x*TILE) +TILE, (y*TILE) + TILE, TILE, TILE, TILE*BADFLAG, 0);
            else if(grille[x][y].hidden)
                wiz_DisplayImagePart(IMG_TILESET, (x*TILE) +TILE, (y*TILE) + TILE, TILE, TILE, TILE*CLOSE, 0);
            else//*/
                wiz_DisplayImagePart(IMG_TILESET, (x*TILE) +TILE, (y*TILE) + TILE, TILE, TILE, TILE*grille[x][y].contain, 0);
        }
    }
}

/* Initialize matrix */
void genere_grille(const int taille, cellule grille[][taille], int n)
{
    int i = 0, x = 0, y = 0;
    
    srand(time(NULL));
    
    for(y = 0; y < taille; y++)
    {
        for(x = 0; x < taille; x++)
        {   // Set default
            grille[x][y].hidden = true;
            grille[x][y].flag = false; 
            grille[x][y].contain = NONE;
        }
    }
    
    i = 0;
    while(i < n)
    {
        x = rand()%taille;
        y = rand()%taille;
        if(grille[x][y].contain != BOMB)
        {
            grille[x][y].contain = BOMB;
            i++;
            increment_voisin(taille, grille, x, y);
        }
    }
    
    /*for(y = 0; y < taille; y++) //for debug
    {
        for(x = 0; x < taille; x++)
        {
            //fprintf(stderr, "%d ", grille[x][y].contain);
        }
        //fprintf(stderr, "\n");
    }//*/
}

void increment_voisin(const int taille, cellule grille[][taille], const int x, const int y)
{
    Uint8 i = 0, j = 0; 
    Sint8 offsetx = -1, offsety = -1;
    
    if(x == 0)
        offsetx = 0;
    if(y == 0)
        offsety = 0;
    
    for(i = y+offsety; i <= y+1; i++)
    {
        if(i < T_GRILLE)
        {
            for(j = x+offsetx; j <= x+1; j++)
            {
                if((j < T_GRILLE) && (grille[j][i].contain != BOMB))
                    grille[j][i].contain++;
            }
        }
    }
}

/* Place the cursor */
void update_selector(Coord *actualpos, Input *in)
{
    Coord tmp = (*actualpos);
    actualpos->x += in->key[VK_RIGHT] - in->key[VK_LEFT];
    actualpos->y += in->key[VK_DOWN] - in->key[VK_UP];
    if(in->mousebuttons[0])
    {
        actualpos->x = ((in->mousex) / 20)-1;
        actualpos->y = ((in->mousey) / 20)-1;
	}
    in->key[VK_RIGHT] = 0;
    in->key[VK_LEFT] = 0;
    in->key[VK_DOWN] = 0;
    in->key[VK_UP] = 0;
    if(actualpos->x >= T_GRILLE || actualpos->y >= T_GRILLE)
    {
        actualpos->x = tmp.x;
        actualpos->y = tmp.y;        
    }
    wiz_DisplayImagePart(IMG_TILESET, (actualpos->x*TILE) +TILE, (actualpos->y*TILE) + TILE, TILE, TILE, TILE*SELECTOR, 0);
}

/* Test a cell */
void tester_cellule(const int taille, cellule grille[][taille], Coord *actualpos,  GameState *state, Uint32 nbrBomb)
{
    if(grille[actualpos->x][actualpos->y].contain == BOMB)
    {
        grille[actualpos->x][actualpos->y].contain = BOOM;
        state->end = true;
        state->win = false;
    }
    else
    {
        devoile_cellule(taille, grille, actualpos, state);
        if(state->cellulesDevoilees == (NBR_CASES) - nbrBomb)
        {
            state->end = true;
            state->win = true;
        }
    }
}

/* open cell */
void devoile_cellule(const int taille, cellule grille[][taille], Coord *actualpos,  GameState *state)
{
    grille[actualpos->x][actualpos->y].hidden = false;
    (state->cellulesDevoilees)++;
    if(grille[actualpos->x][actualpos->y].contain == NONE)
    {
        Coord tmp;
        Sint8 offsetx = -1, offsety = -1;
        
        if(actualpos->x == 0)
            offsetx = 0;
        if(actualpos->y == 0)
            offsety = 0;
            
        for(tmp.y = ((actualpos->y)+offsety); tmp.y <= ((actualpos->y)+1); (tmp.y)++)
        {
            for(tmp.x = ((actualpos->x)+offsetx); tmp.x <= ((actualpos->x)+1); (tmp.x)++)
            {
                if((tmp.x < taille) && (tmp.y < taille) && (grille[tmp.x][tmp.y].hidden) && (!grille[tmp.x][tmp.y].flag))
                    devoile_cellule(taille, grille, &tmp, state);
            }
        }
    }
}

/* Place flag */
void place_flag(const int taille, cellule grille[][taille], Coord *actualpos,  GameState *state) // s'occupe de placer ou de retirer un drapeau
{
    if(grille[actualpos->x][actualpos->y].flag) // if a flag is already placed
    {
        state->reserveFlags++;
        grille[actualpos->x][actualpos->y].flag = !grille[actualpos->x][actualpos->y].flag;
    }
    else // if no flag
    {
        if(state->reserveFlags > 0)
        {
            state->reserveFlags--;
            grille[actualpos->x][actualpos->y].flag = !grille[actualpos->x][actualpos->y].flag;
        }
    }    
}

/* Open all cells */
void open_all(const int taille, cellule grille[][taille])
{
    Coord pos;
    for(pos.y = 0; pos.y < T_GRILLE; pos.y++)
    {
        for(pos.x = 0; pos.x < T_GRILLE; pos.x++)
        {
            grille[pos.x][pos.y].hidden = false;
        }
    }
}

/* Save */
void save_score(GameState *state, Uint32 newScore, Uint32 score[], Uint32 nbrBomb)
{
    FILE *file = NULL;
    Uint8 i, tmp;
    
    state->gamesaved = true; //pour ne plus rentrer dans la condition
    
    for(i = 0; i < 3; i++)
    {
        if(score[i] == 0 || score[i] > newScore)
        {
            tmp = score[i];
            score[i] = newScore;
            newScore = tmp;
        }
    }
    
    if((file = fopen("config.ini", "w")) != NULL) //si le fichier existe
    {
        fprintf(file, "%ld %ld %ld %ld", nbrBomb, score[0], score[1], score[2]);
        fclose(file);
    }    
}






